'use strict';

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = batch;

var _isPlainObj = require('is-plain-obj');

var _isPlainObj2 = _interopRequireDefault(_isPlainObj);

var _isSafari = require('is-safari');

var _isSafari2 = _interopRequireDefault(_isSafari);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Links to array prototype methods.
 */

var slice = [].slice;
var map = [].map;

/**
 * Perform batch operation using `ops`.
 *
 * Array syntax:
 *
 * [
 *   { type: 'add', key: 'key1', val: 'val1' },
 *   { type: 'put', key: 'key2', val: 'val2' },
 *   { type: 'del', key: 'key3' },
 * ]
 *
 * Object syntax:
 *
 * {
 * 	 key1: 'val1', // put val1 to key1
 * 	 key2: 'val2', // put val2 to key2
 * 	 key3: null,   // delete key
 * }
 *
 * @param {Array|Object} ops
 * @return {Promise}
 */

function batch(db, storeName, ops) {
  if (arguments.length !== 3) throw new TypeError('invalid arguments length');
  if (typeof storeName !== 'string') throw new TypeError('invalid "storeName"');
  if (!Array.isArray(ops) && !(0, _isPlainObj2.default)(ops)) throw new TypeError('invalid "ops"');
  if ((0, _isPlainObj2.default)(ops)) {
    ops = Object.keys(ops).map(function (key) {
      return { key: key, value: ops[key], type: ops[key] === null ? 'del' : 'put' };
    });
  }
  ops.forEach(function (op) {
    if (!(0, _isPlainObj2.default)(op)) throw new TypeError('invalid op');
    if (['add', 'put', 'del'].indexOf(op.type) === -1) throw new TypeError('invalid type "' + op.type + '"');
  });

  return new Promise(function (resolve, reject) {
    var tr = db.transaction(storeName, 'readwrite');
    var store = tr.objectStore(storeName);
    var results = [];
    var currentIndex = 0;

    tr.onerror = handleError(reject);
    tr.oncomplete = function () {
      return resolve(results);
    };
    next();

    function next() {
      var _ops$currentIndex = ops[currentIndex];
      var type = _ops$currentIndex.type;
      var key = _ops$currentIndex.key;

      if (type === 'del') return request(store.delete(key));

      var val = ops[currentIndex].val || ops[currentIndex].value;
      if (key && store.keyPath) val[store.keyPath] = key;

      countUniqueIndexes(store, key, val, function (err, uniqueRecordsCounter) {
        if (err) return reject(err);

        // we don't abort transaction here, and just stops execution
        // browsers implementation also don't abort, and just throw an error
        if (uniqueRecordsCounter) return reject(new Error('Unique index ConstraintError'));
        request(store.keyPath ? store[type](val) : store[type](val, key));
      });
    }

    function request(req) {
      currentIndex += 1;

      req.onerror = handleError(reject);
      req.onsuccess = function (e) {
        results.push(e.target.result);
        if (currentIndex < ops.length) next();
      };
    }
  });
}

/**
 * Validate unique index manually.
 *
 * Fixing:
 * - https://bugs.webkit.org/show_bug.cgi?id=149107
 * - https://github.com/axemclion/IndexedDBShim/issues/56
 *
 * @param {IDBStore} store
 * @param {Any} val
 * @param {Function} cb(err, uniqueRecordsCounter)
 */

function countUniqueIndexes(store, key, val, cb) {
  // rely on native support
  if (!_isSafari2.default && global.indexedDB !== global.shimIndexedDB) return cb();

  var indexes = slice.call(store.indexNames).map(function (indexName) {
    var index = store.index(indexName);
    var indexVal = isCompound(index) ? map.call(index.keyPath, function (indexKey) {
      return val[indexKey];
    }).filter(function (v) {
      return Boolean(v);
    }) : val[index.keyPath];

    return [index, indexVal];
  }).filter(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2);

    var index = _ref2[0];
    var indexVal = _ref2[1];

    return index.unique && (isCompound(index) ? indexVal.length : indexVal);
  });

  if (!indexes.length) return cb();

  var totalRequestsCounter = indexes.length;
  var uniqueRecordsCounter = 0;

  indexes.forEach(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2);

    var index = _ref4[0];
    var indexVal = _ref4[1];

    var req = index.getKey(indexVal); // get primaryKey to compare with updating value
    req.onerror = handleError(cb);
    req.onsuccess = function (e) {
      if (e.target.result && e.target.result !== key) uniqueRecordsCounter += 1;
      totalRequestsCounter -= 1;
      if (totalRequestsCounter === 0) cb(null, uniqueRecordsCounter);
    };
  });
}

/**
 * Check if `index` is compound
 *
 * @param {IDBIndex} index
 * @return {Boolean}
 */

function isCompound(index) {
  return typeof index.keyPath !== 'string';
}

/**
 * Create error handler.
 *
 * @param {Function} cb
 * @return {Function}
 */

function handleError(cb) {
  return function (e) {
    // prevent global error throw https://bugzilla.mozilla.org/show_bug.cgi?id=872873
    if (typeof e.preventDefault === 'function') e.preventDefault();
    cb(e.target.error);
  };
}
module.exports = exports['default'];